﻿// BenLincoln.TheLostWorlds.CDHashes
// Copyright 2018 Ben Lincoln
// https://www.thelostworlds.net/
//

// This file is part of BenLincoln.TheLostWorlds.CDHashes.

// BenLincoln.TheLostWorlds.CDHashes is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.TheLostWorlds.CDHashes is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.TheLostWorlds.CDHashes (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using BLD = BenLincoln.Data;

namespace BenLincoln.TheLostWorlds.CDHashes
{
    public enum HashType
    {
        Undefined,
        BloodOmen,
        SoulReaverProto1,
        SoulReaverAlpha,
        SoulReaver,
        SoulReaverDreamcast,
        SoulReaver2AirForge,
        SoulReaver2,
        Defiance
    }

    public class CDHash
    {
        public CDHash()
        {
        }

        public virtual byte[] GetHashBytes(string input, bool bigEndian, bool hashRawText)
        {
            byte[] result = GetHashBytes(input, hashRawText);
            // this method will most likely need to be overridden if the hash length is > 4 bytes
            if (bigEndian)
            {
                result = BLD.EndianConverter.SwapBytes(result, BLD.ByteSwapMode.StraightReverse);
            }
            return result;
        }

        public virtual byte[] GetHashBytes(string input, bool hashRawText)
        {
            return new byte[] { };
        }

        protected byte[] UIntToByteArray(uint inputUInt)
        {
            byte[] result = new byte[4];

            uint ru1 = inputUInt & 0xFF000000;
            uint ru2 = inputUInt & 0x00FF0000;
            uint ru3 = inputUInt & 0x0000FF00;
            uint ru4 = inputUInt & 0x000000FF;

            result[0] = (byte)(ru1 >> 24);
            result[1] = (byte)(ru2 >> 16);
            result[2] = (byte)(ru3 >> 8);
            result[3] = (byte)(ru4);

            return result;
        }

        protected byte[] LongToByteArray(long inputLong)
        {
            byte[] result = new byte[4];

            long ru1 = inputLong & 0xFF000000;
            long ru2 = inputLong & 0x00FF0000;
            long ru3 = inputLong & 0x0000FF00;
            long ru4 = inputLong & 0x000000FF;

            result[0] = (byte)(ru1 >> 24);
            result[1] = (byte)(ru2 >> 16);
            result[2] = (byte)(ru3 >> 8);
            result[3] = (byte)(ru4);

            return result;
        }
    }
}
