﻿// BenLincoln.TheLostWorlds.CDHashes
// Copyright 2018 Ben Lincoln
// https://www.thelostworlds.net/
//

// This file is part of BenLincoln.TheLostWorlds.CDHashes.

// BenLincoln.TheLostWorlds.CDHashes is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.TheLostWorlds.CDHashes is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.TheLostWorlds.CDHashes (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

/* Hashing algorithm used in all versions of Soul Reaver 2 */

namespace BenLincoln.TheLostWorlds.CDHashes
{
    public class SoulReaver2Hash : CDHash
    {
        public SoulReaver2Hash()
            : base()
        {
        }

        public override byte[] GetHashBytes(string input, bool hashRawText)
        {
            char[] inChars, tempChars;
            uint a0, a2, v0, v1, s4, s6, s7;
            uint nameHash;
            int startVal = -1;
            string inVal = input;

            if (hashRawText)
            {
                startVal = 0;
            }
            else
            {
                string extension = inVal.Substring(inVal.LastIndexOf('.') + 1).ToLower();
                string[] extensions = { "drm", "vrm", "raw", "mus", "sam", "smf", "snf", "str" };
                for (int index = 0; index < extensions.Length; index++)
                {
                    if (extension.Equals(extensions[index]))
                    {
                        startVal = index;
                        break;
                    }
                }
                if (startVal == -1)
                {
                    startVal = 0;
                }
                else
                {
                    inVal = inVal.Substring(0, inVal.LastIndexOf('.'));
                }
            }

            inChars = inVal.ToCharArray();
            a0 = 0;
            a2 = 0;
            v0 = 0;
            v1 = 0;
            s4 = 0;
            s6 = 0;
            s7 = (uint)startVal;
            nameHash = 0;
            tempChars = new char[inChars.Length];
            for (int i = 0; i < inChars.Length; i++)
            {
                tempChars[inChars.GetUpperBound(0) - i] = inChars[i];
            }

            inChars = tempChars;

            foreach (char currentChar in inChars)
            {
                v0 = (uint)currentChar;
                v1 = (uint)(v0 - 97);
                if ((v0 > 96) && (v0 < 123))
                {
                    v0 -= 32;
                }

                a0 = s4 << 3;
                a0 = a0 - s4;
                a0 += 3;
                v0 -= 26;
                s4 += 1;
                v0 = v0 << (int)a0;
                if (s4 < 4)
                {
                    v1 = 1;
                }
                else
                {
                    v1 = 0;
                }
                s6 = s6 ^ v0;
                if (v1 == 0)
                {
                    s4 = 0;
                }
            }

            nameHash = (uint)s7 | (uint)s6;

            return UIntToByteArray(nameHash);
        }
    }
}
