﻿// BenLincoln.TheLostWorlds.CDHashes
// Copyright 2018 Ben Lincoln
// https://www.thelostworlds.net/
//

// This file is part of BenLincoln.TheLostWorlds.CDHashes.

// BenLincoln.TheLostWorlds.CDHashes is free software: you can redistribute it and/or modify
// it under the terms of version 3 of the GNU General Public License as published by
// the Free Software Foundation.

// BenLincoln.TheLostWorlds.CDHashes is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with BenLincoln.TheLostWorlds.CDHashes (in the file LICENSE.txt).  
// If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

/* Hashing algorithm used in all versions of Soul Reaver _except_ the Dreamcast release, the proto1 demo, and the alpha builds */

namespace BenLincoln.TheLostWorlds.CDHashes
{
    public class SoulReaverHash : CDHash
    {
        public SoulReaverHash()
            : base()
        {
        }

        public override byte[] GetHashBytes(string input, bool hashRawText)
        {
            string inVal = input;

            uint charsRead = 0;
            uint x1 = 0, x2 = 0;
            int currentLetter = 0;
            int index = 0;
            int extID = 0;
            uint hashName = 0;
            if (hashRawText)
            {
                index = inVal.Length - 1;
            }
            else
            {
                string extension = inVal.Substring(inVal.LastIndexOf('.') + 1).ToLower();
                string[] extensions = { "pcm", "crm", "tim", "smp", "snd", "smf", "snf" };
                for (index = 0; index < extensions.Length; index++)
                {
                    if (extension.Equals(extensions[index]))
                    {
                        extID = index;
                        break;
                    }
                }
                if (index < 7)
                {
                    index = inVal.Length - 5;
                }
                else
                {
                    index = inVal.Length - 1;
                }
            }
            while (index >= 0)
            {
                currentLetter = (int)inVal[index];
                if (currentLetter >= 'a' && currentLetter <= 'z')
                {
                    currentLetter &= 0xDF;
                }
                if (currentLetter == '\\')
                {
                    index--;
                    continue;
                }
                currentLetter += 0xE6;
                currentLetter &= 0xFF;
                x1 = (uint)charsRead;
                x1 *= (uint)currentLetter;
                x2 += (uint)currentLetter;
                hashName ^= x1;
                charsRead++;
                index--;
            }
            charsRead <<= 0x0C;
            charsRead |= x2;
            charsRead <<= 0x0C;
            hashName |= charsRead;
            hashName <<= 0x03;
            hashName |= (uint)extID;

            return UIntToByteArray(hashName);
        }
    }
}
